

#include "SkillProviderExample.h"

#include <RobotAPI/components/skills/SkillProviderExample/aron/HelloWorldAcceptedType.aron.generated.h>
#include <RobotAPI/libraries/aron/converter/json/NLohmannJSONConverter.h>
#include <RobotAPI/libraries/aron/core/type/variant/container/Object.h>
#include <RobotAPI/libraries/aron/core/type/variant/primitive/String.h>

namespace armarx::skills::provider
{
    HelloWorldSkill::HelloWorldSkill() : Skill(GetSkillDescription())
    {
    }

    SkillDescription
    HelloWorldSkill::GetSkillDescription()
    {
        armarx::skills::Example::HelloWorldAcceptedType root_profile_params;
        root_profile_params.some_float = 5;
        root_profile_params.some_int = 42;
        root_profile_params.some_text = "YOLO";
        root_profile_params.some_list_of_matrices.push_back(Eigen::Matrix3f::Zero());
        //default_params.some_matrix = Eigen::Matrix3f::Zero();

        return SkillDescription{{"HelloWorld"},
                                "This skill logs a message on ARMARX_IMPORTANT",
                                root_profile_params.toAron(),
                                armarx::core::time::Duration::MilliSeconds(1000),
                                armarx::skills::Example::HelloWorldAcceptedType::ToAronType()};
    }

    Skill::MainResult
    HelloWorldSkill::main()
    {
        ARMARX_IMPORTANT << "Hi, from the Hello World Skill.\n"
                         << "I received the following data: \n"
                         << aron::data::converter::AronNlohmannJSONConverter::ConvertToNlohmannJSON(
                                parameters)
                                .dump(2)
                         << "\n"
                         << "Type fulfilled? "
                         << parameters->fullfillsType(
                                armarx::skills::Example::HelloWorldAcceptedType::ToAronType())
                         << "\n"
                         << "(executed at: " << IceUtil::Time::now() << ")";
        return {TerminatedSkillStatus::Succeeded, nullptr};
    }

    IncompleteSkill::IncompleteSkill() : Skill(GetSkillDescription())
    {
    }

    SkillDescription
    IncompleteSkill::GetSkillDescription()
    {
        auto d = HelloWorldSkill::GetSkillDescription();
        return SkillDescription{{"IncompleteSkill"},
                                d.description,
                                {},
                                d.timeout + armarx::core::time::Duration::Seconds(2),
                                d.acceptedType};
    }

    Skill::PrepareResult
    IncompleteSkill::prepare()
    {
        if (!first_prepared)
        {
            first_prepared = true;
            std::thread foo(
                [&]()
                {
                    auto d = HelloWorldSkill::GetSkillDescription();
                    std::this_thread::sleep_for(std::chrono::milliseconds(2000));
                    this->setParameters(d.rootProfileParameterization);
                });
            foo.detach();
        }

        auto s = HelloWorldSkill();
        s.setParameters(this->getParameters());
        s.mainOfSkill();

        return {.status = ActiveOrTerminatedSkillStatus::Succeeded};
    }

    Skill::MainResult
    IncompleteSkill::main()
    {
        auto s = HelloWorldSkill();
        s.setParameters(this->getParameters());
        return s.mainOfSkill();
    }

    ChainingSkill::ChainingSkill() : Skill(GetSkillDescription())
    {
    }

    SkillDescription
    ChainingSkill::GetSkillDescription()
    {
        return SkillDescription{{"ChainingSkill"},
                                "This skill calls the HelloWorld skill three times.",
                                {},
                                armarx::core::time::Duration::MilliSeconds(3000),
                                nullptr};
    }

    Skill::MainResult
    ChainingSkill::main()
    {
        armarx::skills::Example::HelloWorldAcceptedType exec1;
        armarx::skills::Example::HelloWorldAcceptedType exec2;
        armarx::skills::Example::HelloWorldAcceptedType exec3;

        exec1.some_text = "Hello from the ChainingSkill 1";
        exec2.some_text = "Hello from the ChainingSkill 2";
        exec3.some_text = "Hello from the ChainingSkill 3";

        SkillProxy skillExecPrx(
            manager, skills::SkillID(skills::ProviderID("SkillProviderExample"), "HelloWorld"));

        skillExecPrx.executeSkill(getSkillId().toString(), exec1.toAron());
        skillExecPrx.executeSkill(getSkillId().toString(), exec2.toAron());
        skillExecPrx.executeSkill(getSkillId().toString(), exec3.toAron());

        return {TerminatedSkillStatus::Succeeded, nullptr};
    }

    TimeoutSkill::TimeoutSkill() :
        PeriodicSkill(GetSkillDescription(), armarx::core::time::Frequency::Hertz(5))
    {
    }

    SkillDescription
    TimeoutSkill::GetSkillDescription()
    {
        return SkillDescription{{"Timeout"},
                                "This fails with timeout reached",
                                {},
                                armarx::core::time::Duration::MilliSeconds(5000),
                                nullptr};
    }

    PeriodicSkill::StepResult
    TimeoutSkill::step()
    {
        // do heavy work
        std::this_thread::sleep_for(std::chrono::milliseconds(500));

        return {ActiveOrTerminatedSkillStatus::Running, nullptr};
    }

    CallbackSkill::CallbackSkill() : Skill(GetSkillDescription())
    {
    }

    SkillDescription
    CallbackSkill::GetSkillDescription()
    {
        return SkillDescription{{"ShowMeCallbacks"},
                                "This skill does shows callbacks",
                                {},
                                armarx::core::time::Duration::MilliSeconds(1000),
                                nullptr};
    }

    Skill::MainResult
    CallbackSkill::main()
    {
        ARMARX_IMPORTANT << "Logging three updates via the callback";
        auto up1 = std::make_shared<aron::data::Dict>();
        up1->addElement("updateInfo", std::make_shared<aron::data::String>("Update 1"));

        this->callback(skills::SkillStatus::Running, up1);

        auto up2 = std::make_shared<aron::data::Dict>();
        up2->addElement("updateInfo", std::make_shared<aron::data::String>("Update 2"));
        this->callback(skills::SkillStatus::Running, up2);

        auto up3 = std::make_shared<aron::data::Dict>();
        up3->addElement("updateInfo", std::make_shared<aron::data::String>("Update 3"));
        this->callback(skills::SkillStatus::Running, up3);

        return {TerminatedSkillStatus::Succeeded, nullptr};
    }

    SkillProviderExample::SkillProviderExample() : SkillProviderComponentPluginUser()
    {
    }

    armarx::PropertyDefinitionsPtr
    SkillProviderExample::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());
        return defs;
    }

    std::string
    SkillProviderExample::getDefaultName() const
    {
        return "SkillProviderExample";
    }

    void
    SkillProviderExample::onInitComponent()
    {
        // Add example skill
        ARMARX_INFO << "Adding skill HelloWorldSkill";
        addSkillFactory<HelloWorldSkill>();

        // Add another lambda example skill
        {
            skills::SkillDescription fooDesc({"Foo"},
                                             "This skill does exactly nothing.",
                                             nullptr,
                                             armarx::core::time::Duration::MilliSeconds(1000),
                                             nullptr);
            addSkillFactory(fooDesc,
                            []()
                            {
                                std::cout << "Hello from Foo." << std::endl;
                                return TerminatedSkillStatus::Succeeded;
                            });
        }

        // Add another example skill
        ARMARX_INFO << "Adding skill CallbackSkill";
        addSkillFactory<CallbackSkill>();

        // Add timeout skill
        ARMARX_INFO << "Adding skill TimeoutSkill";
        addSkillFactory<TimeoutSkill>();

        // chaining
        ARMARX_INFO << "Adding skill ChainingSkill";
        addSkillFactory<ChainingSkill>();

        // incomplete and prepare
        ARMARX_INFO << "Adding skill IncompleteSkill";
        addSkillFactory<IncompleteSkill>();
    }

    void
    SkillProviderExample::onConnectComponent()
    {
    }

    void
    SkillProviderExample::onDisconnectComponent()
    {
    }

    void
    SkillProviderExample::onExitComponent()
    {
    }
} // namespace armarx::skills::provider
