/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>

#include <VirtualRobot/VirtualRobot.h>

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/services/tasks/TaskUtil.h>
#include <ArmarXCore/core/time.h>
#include <ArmarXCore/libraries/DebugObserverHelper/DebugObserverHelper.h>

#include "RobotAPI/libraries/armem/server/wm/memory_definitions.h"
#include "RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h"
#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/libraries/armem/server/segment/SpecializedCoreSegment.h>
#include <RobotAPI/libraries/armem_laser_scans/types.h>
#include <RobotAPI/libraries/armem_objects/types.h>
#include <RobotAPI/libraries/armem_robot_state/server/forward_declarations.h>

namespace armarx::armem::server::laser_scans
{
    struct ScanPoint
    {
        Eigen::Vector3f point;
        float intensity;
    };

    /**
     * @brief Models decay of object localizations by decreasing the confidence
     * the longer the object was not localized.
     */
    class Visu : public armarx::Logging
    {
    public:
        Visu();


        void defineProperties(armarx::PropertyDefinitionsPtr defs,
                              const std::string& prefix = "visu.");
        void init(const wm::CoreSegment* coreSegment,
                  armem::robot_state::VirtualRobotReader* virtualRobotReader);
        void connect(const viz::Client& arviz, DebugObserverInterfacePrx debugObserver = nullptr);


    private:
        void visualizeRun();
        void visualizeOnce(const Time& timestamp);


        viz::Client arviz;
        std::optional<DebugObserverHelper> debugObserver;

        struct Properties
        {
            bool enabled = true;
            bool uniformColor = false;
            bool colorByIntensity = true;
            float frequencyHz = 5;
            int maxRobotAgeMs = 100;
        } p;

        SimpleRunningTask<>::pointer_type updateTask;

        armem::robot_state::VirtualRobotReader* virtualRobotReader;

        std::map<std::string, std::pair<VirtualRobot::RobotPtr, DateTime>> robots;

        VirtualRobot::RobotPtr getSynchronizedRobot(const std::string& name,
                                                    const DateTime& timestamp);

        void visualizeScan(const std::vector<ScanPoint>& points,
                           const std::string& sensorName,
                           const std::string& agentName,
                           const viz::Color& color);

        std::map<std::string, armem::laser_scans::LaserScanStamped> getCurrentLaserScans();

        const wm::CoreSegment* coreSegment;
    };

} // namespace armarx::armem::server::laser_scans
