#include "SkillEventSegment.h"

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <RobotAPI/libraries/armem_skills/aron/Skill.aron.generated.h>
#include <RobotAPI/libraries/armem_skills/aron_conversions.h>
#include <RobotAPI/libraries/aron/converter/json/NLohmannJSONConverter.h>

namespace armarx::skills::segment
{

    SkillEventCoreSegment::SkillEventCoreSegment(armem::server::MemoryToIceAdapter& iceMemory) :
        Base(iceMemory,
             CoreSegmentName,
             armarx::skills::arondto::SkillExecutionRequest::ToAronType())
    {
    }

    void
    SkillEventCoreSegment::defineProperties(PropertyDefinitionsPtr defs, const std::string& prefix)
    {
        // No properties! (meaning no name and no max size)
    }

    void
    SkillEventCoreSegment::init()
    {
        Base::init();
    }

    void
    SkillEventCoreSegment::addSkillUpdateEvent(const skills::SkillStatusUpdate& update)
    {
        // create commit about new update
        armarx::skills::arondto::SkillExecutionEvent event;
        armem::toAron(event, update);

        armem::MemoryID commitId = id();
        commitId.providerSegmentName = event.skillId.providerId.providerName;
        commitId.entityName = event.skillId.skillName;

        auto aron = event.toAron();

        armem::Commit comm;
        auto& entityUpdate = comm.add();
        entityUpdate.confidence = 1.0;
        entityUpdate.referencedTime = armem::Time::Now();
        entityUpdate.instancesData = {aron};
        entityUpdate.entityID = commitId;

        iceMemory.commitLocking(comm);
    }

    std::map<skills::SkillExecutionID, skills::SkillStatusUpdate>
    SkillEventCoreSegment::getLatestSkillEvents(int n)
    {
        std::map<skills::SkillExecutionID, skills::SkillStatusUpdate> ret;
        auto coreSegment = this->segmentPtr;
        ARMARX_CHECK(coreSegment);

        // 1. get all events and sort them by timestamp.
        std::vector<skills::SkillStatusUpdate> sorted;
        coreSegment->forEachInstance(
            [&sorted](const armem::wm::EntityInstance& i)
            {
                auto event = i.dataAs<armarx::skills::arondto::SkillExecutionEvent>();
                skills::SkillStatusUpdate up({{{""}, ""}, "", armarx::core::time::DateTime::Now()},
                                             {nullptr, nullptr});
                armem::fromAron(event, up);
                sorted.emplace_back(std::move(up));
            });

        std::sort(sorted.begin(),
                  sorted.end(),
                  [](const skills::SkillStatusUpdate& a, const skills::SkillStatusUpdate& b)
                  { return b < a; });

        for (const auto& el : sorted)
        {
            if (ret.size() >= (size_t)n)
            {
                break;
            }

            ret.emplace(
                std::piecewise_construct, std::make_tuple(el.executionId), std::make_tuple(el));
        }

        return ret;
    }
} // namespace armarx::skills::segment
