#pragma once

#include <string>
#include <vector>

#include <SimoxUtility/algorithm/string.h>

#include <ArmarXCore/core/time/DateTime.h>
#include <ArmarXCore/core/time/ice_conversions.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "SkillID.h"
#include "error/Exception.h"

namespace armarx
{
    namespace skills
    {
        class SkillExecutionRequest
        {
        public:
            skills::SkillID skillId;
            std::string executorName;
            armarx::aron::data::DictPtr params;
            callback::dti::SkillProviderCallbackInterfacePrx callbackInterface;

            SkillExecutionRequest() = delete;
            SkillExecutionRequest(
                const skills::SkillID& skill,
                const std::string executorName,
                const armarx::aron::data::DictPtr& params,
                const callback::dti::SkillProviderCallbackInterfacePrx& callback = nullptr);

            manager::dto::SkillExecutionRequest toManagerIce() const;
            provider::dto::SkillExecutionRequest toProviderIce() const;

            static SkillExecutionRequest FromIce(const manager::dto::SkillExecutionRequest&);
            static SkillExecutionRequest
            FromIce(const provider::dto::SkillExecutionRequest&,
                    const std::optional<skills::ProviderID>& providerId = std::nullopt);
        };
    } // namespace skills
} // namespace armarx
