#pragma once

#include <RobotAPI/libraries/skills/core/Skill.h>

namespace armarx
{
    namespace skills
    {
        template <class AronT>
        class SimpleSpecializedSkill : public Skill
        {
        public:
            using ParamType = AronT;
            using Skill::Skill;

            struct SpecializedInitInput
            {
                std::string executorName;
                AronT params;
            };

            struct SpecializedMainInput
            {
                std::string executorName;
                AronT params;
                CallbackT callback;
            };

            struct SpecializedExitInput
            {
                std::string executorName;
                AronT params;
            };


        protected:
            // legacy methods
            virtual InitResult
            init(const SpecializedInitInput& in)
            {
                return InitResult{.status = TerminatedSkillStatus::Succeeded};
            }

            virtual MainResult
            main(const SpecializedMainInput& in)
            {
                ARMARX_IMPORTANT << "Dummy executing skill '" << description.skillId
                                 << "'. Please overwrite this method.";
                return Skill::MainResult{.status = TerminatedSkillStatus::Succeeded,
                                         .data = nullptr};
            }

            virtual ExitResult
            exit(const SpecializedExitInput& in)
            {
                return ExitResult{.status = TerminatedSkillStatus::Succeeded};
            }

            InitResult
            init() final
            {
                AronT p;
                p.fromAron(this->parameters);

                SpecializedInitInput i;
                i.executorName = this->executorName;
                i.params = p;
                return this->init(i);
            }

            MainResult
            main() final
            {
                AronT p;
                p.fromAron(this->parameters);

                SpecializedMainInput i;
                i.executorName = this->executorName;
                i.callback = this->callback;
                i.params = p;
                return this->main(i);
            }

            ExitResult
            exit() final
            {
                AronT p;
                p.fromAron(this->parameters);

                SpecializedExitInput i;
                i.executorName = this->executorName;
                i.params = p;
                return this->exit(i);
            }
        };
    } // namespace skills
} // namespace armarx
