#include "SkillProxy.h"

namespace armarx
{
    namespace skills
    {
        SkillProxy::SkillProxy(const manager::dti::SkillManagerInterfacePrx& manager,
                               const SkillID& skillId) :
            manager(manager),
            skillDescription(SkillDescription::FromIce(
                manager->getSkillDescription(skillId.toManagerIce()).value()))
        {
        }

        SkillProxy::SkillProxy(const manager::dti::SkillManagerInterfacePrx& manager,
                               const SkillDescription& skillDesc) :
            manager(manager), skillDescription(skillDesc)
        {
            ARMARX_CHECK(skillDesc.skillId.fullySpecified());
        }

        TerminatedSkillStatusUpdate
        SkillProxy::executeSkill(const std::string& executorName, const aron::data::DictPtr& params)
        {
            skills::manager::dto::SkillExecutionRequest req;
            req.executorName = executorName;
            req.params = params->toAronDictDTO();
            req.skillId = skillDescription.skillId.toManagerIce();

            auto terminatingUpdateIce = manager->executeSkill(req);
            return TerminatedSkillStatusUpdate::FromIce(terminatingUpdateIce);
        }

        SkillExecutionID
        SkillProxy::executeSkillAsync(const std::string& executorName,
                                      const aron::data::DictPtr& params)
        {
            skills::manager::dto::SkillExecutionRequest req;
            req.executorName = executorName;
            req.params = params->toAronDictDTO();
            req.skillId = skillDescription.skillId.toManagerIce();

            auto execReqIce = manager->executeSkillAsync(req);
            return SkillExecutionID::FromIce(execReqIce);
        }

        void
        SkillProxy::abortSkill(const SkillExecutionID& id)
        {
            // TODO: This will be used in the future, do not remove it!
            manager->abortSkill(id.toManagerIce());
        }

        aron::data::DictPtr
        SkillProxy::getDefaultParameters(const std::string& profileName)
        {
            if (profileName == "root")
            {
                return skillDescription.rootProfileParameterization;
            }
            // TODO @fabianPK
            return nullptr;
        }
    } // namespace skills
} // namespace armarx
